#include <p18cxxx.h>
#include <delays.h>
#include "lcd/Nokia6100Driver.h"
#include "lcd/lucida_font.h"


#define abs(a)	   (((a) < 0) ? -(a) : (a))




//********************************************************************
//
//				Global Variables for LCD driver
//
//********************************************************************
//Usage: LCDClear(black);
//Inputs: char color: 8-bit color to be sent to the screen.
//Outputs: None
//Description: This function will clear the screen with "color" by writing the
//			   color to each location in the RAM of the LCD.
void LCDClear(int color)
{
	int i;
	#ifdef EPSON
		LCDCommand(PASET);
		LCDData(0);
		LCDData(131);
	
		LCDCommand(CASET);
		LCDData(0);
		LCDData(131);

		LCDCommand(RAMWR);
	#endif
	#ifdef	PHILLIPS
		LCDCommand(PASETP);
		LCDData(0);
		LCDData(131);
	
		LCDCommand(CASETP);
		LCDData(0);
		LCDData(131);

		LCDCommand(RAMWRP);
	#endif
	for(i=0; i < (132 * 132); i++)
	{
		/*
		LCDData((color>>4)&0x00FF);
		LCDData(((color&0x0F)<<4)|(color>>8));
		LCDData(color&0x0FF);
		*/
		LCDData(color);

		
		
	}
}

//Usage: LCDCommand(RAMWR);
//Inputs: char data - character command to be sent to the LCD
//Outputs: None
//Description: Sends a 9 bit command over SPI to the LCD with
//			   the first bit HIGH indicating "command" mode.
void LCDCommand(unsigned char data)
{
	char j;
		//Assign LCD pins as Outputs		
	LCD_RES_TRIS = 0;
	LCD_CS_TRIS =0;
	LCD_DIO_TRIS =0;
	LCD_SCK_TRIS =0;
	
	LCD_CS = 0;      // enable chip, p0.20 goes low
    LCD_DIO = 0;     // output low on data out (9th bit low = command), p0.19

    LCD_SCK = 0;      // send clock pulse
    LCD_SCK = 1;

    for (j = 0; j < 8; j++)
    {
        if ((data & 0x80) == 0x80) LCD_DIO = 1;
        else LCD_DIO = 0;

        LCD_SCK = 0;   // send clock pulse
        LCD_SCK = 1;

        data <<= 1;
    }

    LCD_CS = 1;    		// disable

}

//Usage: LCDData(RAMWR);
//Inputs: char data - character data to be sent to the LCD
//Outputs: None
//Description: Sends a 9 bit command over SPI to the LCD with
//			   the first bit LOW indicating "data" mode.
void LCDData(unsigned char data)
{
	char j;
	//Assign LCD pins as Outputs		
	LCD_RES_TRIS = 0;
	LCD_CS_TRIS =0;
	LCD_DIO_TRIS =0;
	LCD_SCK_TRIS =0;
	
	//LCD_SCK = 0;      // send clock pulse
	LCD_CS = 0;      // enable chip, p0.20 goes low    
	LCD_DIO = 1;    // output high on data out (9th bit high = data), p0.19

    LCD_SCK = 0;    // send clock pulse
    LCD_SCK = 1;
	for (j = 0; j < 8; j++)
    {
        if ((data & 0x80) == 0x80) LCD_DIO = 1;
        else LCD_DIO = 0;
        LCD_SCK = 0;   // send clock pulse
        LCD_SCK = 1;
        data <<= 1;
    }

    LCD_CS = 1;     		// disable
}

//Usage: LCDInit();
//Inputs: None
//Outputs: None
//Description:  Initializes the LCD regardless of if the controlller is an EPSON or PHILLIPS.
void LCDInit(void)
{
    int j;	
    MADCTLOptions opt;

	Delay10KTCYx(200);
	
    // reset display	
	//Assign LCD pins as Outputs		
	LCD_RES_TRIS = 0;
	LCD_CS_TRIS =0;
	LCD_DIO_TRIS =0;
	LCD_SCK_TRIS =0;
	

	//output_low (SPI_CLK);//output_low (SPI_DO);
	
	LCD_DIO = 0;
	LCD_SCK = 0;
	
	LCD_CS = 1;		//output_high (LCD_CS);			
			
    Delay10TCYx(3);
    
    LCD_RES = 0;				//output_low (LCD_RESET);
    Delay10KTCYx(200);
    Delay10KTCYx(200);
    Delay10KTCYx(200);
    LCD_RES = 1;				//output_high (LCD_RESET);
   
    
    LCD_SCK = 1;
    LCD_DIO = 1;
    Delay10KTCYx(200);	//delay_ms(100);
    Delay10KTCYx(200);
    Delay10KTCYx(200);
	
///    LCDCommand(DISCTL);  	// display control(EPSON)
//    LCDData(0x0C);   		// 12 = 1100 - CL dividing ratio [don't divide] switching period 8H (default)
//	LCDData(0x20);    
//	LCDData(0x00);
	
//	LCDData(0x01);
	
//    LCDCommand(COMSCN);  	// common scanning direction(EPSON)
//    LCDData(0x01);
    
 //   LCDCommand(OSCON);  	// internal oscialltor ON(EPSON)
	
//    LCDCommand(SLPOUT);  	// sleep out(EPSON)
	LCDCommand(SLEEPOUT);	//sleep out(PHILLIPS)
    
  //  LCDCommand(PWRCTR); 	// power ctrl(EPSON)
 //   LCDData(0x0F);    		//everything on, no external reference resistors
    //LCDCommand(BSTRON);		//Booset On(PHILLIPS)
	
//	LCDCommand(DISINV);  	// invert display mode(EPSON)
	LCDCommand(INVOFF);		// invert display mode(PHILLIPS)
    
   // LCDCommand(DATCTL);  	// data control(EPSON)
   // LCDData(0x03);			//correct for normal sin7
//	LCDData(0x00);   		// normal RGB arrangement
//	LCDData(0x02);			// 16-bit Grayscale Type A
	

	
	LCDCommand(COLMOD);		//Set Color Mode(PHILLIPS)
	LCDData(0x02);	
	

	
	
	opt.bits.MY = 1;
	opt.bits.MX = 1;
	opt.bits.V = 0;
	opt.bits.LAO = 0;
	opt.bits.RGB = 0;
	opt.bits.X1 = 0;
	opt.bits.X2 = 0;
	opt.bits.X3 = 0;
	LCDCommand(MADCTL);		//Memory Access Control(PHILLIPS)
	LCDData(opt.MADCTLByte);	

	
	
   // LCDCommand(VOLCTR);  	// electronic volume, this is the contrast/brightness(EPSON)
//	LCDData(0x24);   		// volume (contrast) setting - fine tuning, original
  //  LCDData(0x03);   		// internal resistor ratio - coarse adjustment
	LCDCommand(SETCON);		//Set Contrast(PHILLIPS)
	LCDData(0x30);	
	  Delay10KTCYx(20); 
	   
   // LCDCommand(NOP);  	// nop(EPSON)
	//LCDCommand(NOPP);		// nop(PHILLIPS)

  //  LCDCommand(DISON);   	// display on(EPSON)
	LCDCommand(DISPON);	// display on(PHILLIPS)
}

//Usage: LCDSetPixel(white, 0, 0);
//Inputs: unsigned char color - desired color of the pixel
//		  unsigned char x - Page address of pixel to be colored
//		  unsigned char y - column address of pixel to be colored
//Outputs: None
//Description: Sets the starting page(row) and column (x & y) coordinates in ram,
//  		   then writes the colour to display memory.  The ending x & y are left
//  		   maxed out so one can continue sending colour data bytes to the 'open'
//  		   RAMWR command to fill further memory.  issuing any red command
//  		   finishes RAMWR.
void LCDSetPixel(int color, unsigned char x, unsigned char y)
{
	#ifdef EPSON
		LCDCommand(PASET);   // page start/end ram
		LCDData(x);
		LCDData(ENDPAGE);
  
		LCDCommand(CASET);   // column start/end ram
		LCDData(y);
		LCDData(ENDCOL);
  
		LCDCommand(RAMWR);    // write
		LCDData((color>>4)&0x00FF);
		LCDData(((color&0x0F)<<4)|(color>>8));
		LCDData(color&0x0FF);  	// nop(EPSON)		
	#endif
	#ifdef	PHILLIPS
		LCDCommand(PASETP);   // page start/end ram
		LCDData(x);
		LCDData(ENDPAGE);
  
		LCDCommand(CASETP);   // column start/end ram
		LCDData(y);
		LCDData(ENDCOL);
  
		LCDCommand(RAMWRP);    // write
		LCDData(color);
		/*
		LCDData((unsigned char)((color>>4)&0x00FF));
		LCDData((unsigned char)(((color&0x0F)<<4)|0x00));
		*/
	#endif
}

void drawCircle(int color, int x0, int y0, int radius)
{
	  int f = 1 - radius;
	  int ddF_x = 1;
	  int ddF_y = -2 * radius;
	  int x = 0;
	  int y = radius;
	 
	  LCDSetPixel(color, x0, y0 + radius);
	  LCDSetPixel(color, x0, y0 - radius);
	  LCDSetPixel(color, x0 + radius, y0);
	  LCDSetPixel(color, x0 - radius, y0);
	 
	  while(x < y)
	  {
		if(f >= 0) 
		{
		  y--;
		  ddF_y += 2;
		  f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x;    
		LCDSetPixel(color, x0 + x, y0 + y);
		LCDSetPixel(color, x0 - x, y0 + y);
		LCDSetPixel(color, x0 + x, y0 - y);
		LCDSetPixel(color, x0 - x, y0 - y);
	    LCDSetPixel(color, x0 + y, y0 + x);
		LCDSetPixel(color, x0 - y, y0 + x);
		LCDSetPixel(color, x0 + y, y0 - x);
		LCDSetPixel(color, x0 - y, y0 - x);
	  }
}

void drawFilledCircle(BYTE color, BYTE x0, BYTE y0, BYTE radius)
{
  int f = 1 - radius;
  int ddF_x = 1;
  int ddF_y = -2 * radius;
  int x = 0;
  int y = radius;
  char noy;


 		drawHorizontalLine(color, x0 - radius, y0,x0 + radius);

  while(x < (y-1))
  {

	 noy = 0;
    if(f >= 0)
    {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    else
    {

    		noy = 1;


    }
    x++;
    ddF_x += 2;
    f += ddF_x;

    
 
    		if (!noy)
    		{
    			drawHorizontalLine(color, x0 - x, y0 + y,x0 + x);
    		}
    
		if (y != 0)
		{
		
				if (!noy)
				{
					drawHorizontalLine(color, x0 - x, y0 - y,x0 + x);
				}
		}

    	drawHorizontalLine(color, x0 - y, y0 + x,x0 + y);
    
	if (x != 0)
	{
			drawHorizontalLine(color, x0 - y, y0 - x,x0 + y);
	}
  }
}


void drawBox(BYTE color, BYTE x0, BYTE y0, BYTE x1, BYTE y1, BYTE BorderWeight)
{
	int p;
	BYTE tempVal;
	
	if (x0 > x1)
	{
		tempVal = x0;
		x0 = x1;
		x1 = tempVal;
	}
	
	if (y0 > y1)
	{
		tempVal = y0;
		y0 = y1;
		y1 = tempVal;
	}

	
	LCDSetBoundries(x0, y0+(BorderWeight-1), x0+(BorderWeight-1), y1+(BorderWeight-1));
	for (p = 0; p <= (((y1-y0)+1)*BorderWeight); p++)
	{
		LCDData(color);
	}
	LCDSetBoundries(x1, y0, x1+(BorderWeight-1), y1);
	for (p = 0; p <= (((y1-y0)+1)*BorderWeight); p++)
	{
		LCDData(color);
	}
	LCDSetBoundries(x0, y0, x1, y0+(BorderWeight-1));
	for (p = 0; p <= (((x1-x0)+1)*BorderWeight); p++)
	{
		LCDData(color);
	}
	LCDSetBoundries(x0+(BorderWeight-1), y1, x1+(BorderWeight-1), y1+(BorderWeight-1));
	for (p = 0; p <= (((x1-x0)+1)*BorderWeight); p++)
	{
		LCDData(color);
	}
	
	/*
	for (p = x0; p <= x1; p++)
	{
		LCDSetPixel(color,p,y0);
		LCDSetPixel(color,p,y1);
	}
	for (p = y0; p <= y1; p++)
	{
		LCDSetPixel(color,x0,p);
		LCDSetPixel(color,x1,p);
	}
	*/

}

//Draw a filled box using the feature of the LCD controller that
//allows you to write continuously within specified boundries.

void drawFilledBox(BYTE color, BYTE x0, BYTE y0, BYTE x1, BYTE y1)
{
	int p;
	int TotalPixels;
	int tempVal;
	
	if (x0 > x1)
	{
		tempVal = x0;
		x0 = x1;
		x1 = tempVal;
	}
	
	if (y0 > y1)
	{
		tempVal = y0;
		y0 = y1;
		y1 = tempVal;
	}
	
	LCDSetBoundries(x0, y0, x1, y1);
	TotalPixels = ((y1-y0)+1) * ((x1-x0)+1);	

	for (p = 0; p <= TotalPixels; p++)
	{
	
			LCDData(color);
	}

}

void LCDSetBoundries(BYTE x0, BYTE y0, BYTE x1, BYTE y1)
{
	LCDCommand(PASETP);
	LCDData(y0);
	LCDData(y1);

	LCDCommand(CASETP);
	LCDData(x0);
	LCDData(x1);

	LCDCommand(RAMWRP);
}	


/* 
	This is a custom adaptation of Bresenham's line algorithm
*/
void drawLine(int color, int x0, int y0, int x1, int y1)
{
	int deltax;
	int deltay;
	int error;
	int ystep;
	int y;
	int x;
	unsigned char steep;
	int temp;
	steep = ((abs(y1 - y0)) > (abs(x1 - x0)));
     if (steep)
	 {
         /*swap x0, y0*/
		 temp = x0;
		 x0 = y0;
		 y0 = temp;
		 
         /*swap x1, y1*/
		 temp = x1;
		 x1 = y1;
		 y1 = temp;
	 }
     if (x0 > x1)
	 {
         /*swap x0, x1*/
		 temp = x0;
		 x0 = x1;
		 x1 = temp;
		 
         /*swap y0, y1*/
		 temp = y0;
		 y0 = y1;
		 y1 = temp;
	 }
     deltax = x1 - x0;
     deltay = abs(y1 - y0);
     error = deltax / 2;
     y = y0;
     if (y0 < y1)
	 {
		ystep = 1;
	 }
	 else 
	 {
		ystep = -1;
	 }
     for (x = x0; x <=x1; x++)
	 {
         if (steep)
		 {		
			LCDSetPixel(color, y,x);
		 } 
		 else 
		 {
		    LCDSetPixel(color, x,y);
		 }
         error = error - deltay;
         if (error < 0)
		 {
             y = y + ystep;
             error = error + deltax;
	     }
	 }


}
void drawHorizontalLine(BYTE color, BYTE x0, BYTE y0, BYTE x1)
{
	BYTE tempVal;
	BYTE distance;
	BYTE i;
	
	if (x0 > x1)
	{
		tempVal = x0;
		x0 = x1;
		x1 = tempVal;
	} 
	LCDSetBoundries(x0, y0, x1, y0);
	distance = (x1-x0) + 1;
	for (i = 0; i < distance; i++)
	{
		LCDData(color);	
	}	
	
}	

void LCDString(int Color, int FillColor, char * DisplayString, BYTE x, BYTE y)
{
	BYTE t;
	BYTE p;
	while (*DisplayString)
	{
		unsigned int StartIndex = (*DisplayString - 32) * LUCIDA_CHAR_HEIGHT;
		LCDSetBoundries(x, y, x+(LUCIDA_CHAR_WIDTH), y+(LUCIDA_CHAR_HEIGHT-1));
		for (t = 0; t < (LUCIDA_CHAR_HEIGHT * (LUCIDA_CHAR_WIDTH/8)); t++)
		{
			BYTE LetterLine = *(LUCIDA_FONT + StartIndex + t);
			for (p = 0; p < 8; p++)
			{
				if (LetterLine & (1 << (LUCIDA_CHAR_WIDTH-1-p)))
				{
					LCDData(Color);
				}
				else
				{
					LCDData(FillColor);
				}		
				
			}
			LCDData(FillColor); //Space
		
		}
	
		x+=(LUCIDA_CHAR_WIDTH+1);
		
		if (x > 130) //Wrap around
		{
			x = 0;
			y+=(LUCIDA_CHAR_HEIGHT+1);
		}	
		if (y > 130)
		{
			y = 0;
		}	
		DisplayString++;	
	}
}	